## A New Dawn Mod Code
##

from CvPythonExtensions import *
gc = CyGlobalContext()
import BugOptions
import BugCore
import BugUtil
import Popup as PyPopup
import CvUtil
import PlayerUtil
ANewDawnOpt = BugCore.game.RoMSettings
MainOpt = BugCore.game.MainInterface

MODDEROPTION_EVENT_ID = CvUtil.getNewEventID("Modder Options")
MODDERGAMEOPTION_EVENT_ID = CvUtil.getNewEventID("Modder Game Options")
DIFFICULTY_EVENT_ID = CvUtil.getNewEventID("Difficulty")
COLOR_EVENT_ID = CvUtil.getNewEventID("Color")
CANBUILD_EVENT_ID = CvUtil.getNewEventID("CanBuild")

class ANewDawnSettings:
	def __init__(self, eventManager):
		eventManager.addEventHandler("OnLoad", self.onLoadGame)
		eventManager.addEventHandler("GameStart", self.onGameStart)
		eventManager.addEventHandler("ModNetMessage", self.onModNetMessage)

	def onLoadGame(self,argsList):
		self.optionUpdate()
		MainOpt.setShowOptionsKeyReminder(True)

	def onGameStart(self,argsList):
		self.optionUpdate()
		MainOpt.setShowOptionsKeyReminder(True)

	def optionUpdate(self):	
		if ANewDawnOpt.isRoMReset():
			resetOptions()
		else:
			setXMLOptionsfromIniFile()

	def onModNetMessage(self, argsList):
		protocol, data1, data2, data3, data4 = argsList
		#ModderOptions
		if (protocol == MODDEROPTION_EVENT_ID):
			pPlayer = gc.getPlayer(data1)
			pPlayer.setModderOption(data2, data3)
		#ModderGameOptions
		elif (protocol == MODDERGAMEOPTION_EVENT_ID):
			#Only accept commands from host
			if (data1 == 0):
				if (data2 == int(ModderGameOptionTypes.MODDERGAMEOPTION_DEFENDER_WITHDRAW)):
					ANewDawnOpt.setDefenderWithdraw(data3)
				elif (data2 == int(ModderGameOptionTypes.MODDERGAMEOPTION_MAX_UNITS_PER_TILES)):
					ANewDawnOpt.setMaxUnitsPerTile(data3)
				elif (data2 == int(ModderGameOptionTypes.MODDERGAMEOPTION_MAX_BOMBARD_DEFENSE)):
					ANewDawnOpt.setMaxBombardDefense(data3)
				elif (data2 == int(ModderGameOptionTypes.MODDERGAMEOPTION_CANNOT_CLAIM_OCEAN)):
					ANewDawnOpt.setCanNotClaimOcean(data3)
				elif (data2 == int(ModderGameOptionTypes.MODDERGAMEOPTION_RESOURCE_DEPLETION)):
					ANewDawnOpt.setDepletionMod(data3)
				elif (data2 == int(ModderGameOptionTypes.MODDERGAMEOPTION_BETTER_INTERCETION)):
					ANewDawnOpt.setBetterAirInterception(data3)
				elif (data2 == int(ModderGameOptionTypes.MODDERGAMEOPTION_AIRLIFT_RANGE)):
					ANewDawnOpt.setMaxRebaseRange(data3)
				elif (data2 == int(ModderGameOptionTypes.MODDERGAMEOPTION_MERCY_RULE)):
					ANewDawnOpt.setMercyRule(data3)
				elif (data2 == int(ModderGameOptionTypes.MODDERGAMEOPTION_REALISTIC_DIPLOMACY)):
					ANewDawnOpt.setRealisiticDiplomacy(data3)
				elif (data2 == int(ModderGameOptionTypes.MODDERGAMEOPTION_BATTLEFIELD_PROMOTIONS)):
					ANewDawnOpt.setBattlefieldPromotions(data3)
				elif (data2 == int(ModderGameOptionTypes.MODDERGAMEOPTION_NO_STORMS)):
					ANewDawnOpt.setNoStorms(data3)
				elif (data2 == int(ModderGameOptionTypes.MODDERGAMEOPTION_IMPROVED_XP)):
					ANewDawnOpt.setImprovedXP(data3)
				elif (data2 == int(ModderGameOptionTypes.MODDERGAMEOPTION_MULTIPLE_RELIGION_SPREAD)):
					ANewDawnOpt.setMultipleReligionSpread(data3)
				elif (data2 == int(ModderGameOptionTypes.MODDERGAMEOPTION_TERRAIN_DAMAGE)):
					ANewDawnOpt.setTerrainDamage(data3)
				elif (data2 == int(ModderGameOptionTypes.MODDERGAMEOPTION_STRATEGIC_EVENTS)):
					ANewDawnOpt.setStrategicEvents(data3)
				elif (data2 == int(ModderGameOptionTypes.MODDERGAMEOPTION_NO_AUTO_CORPORATION_FOUNDING)):
					ANewDawnOpt.setNoAutoCorporationFounding(data3)
				#gc.getGame().setModderGameOption(data2, data3)
			else:
				BugUtil.debug("Non-Host Player Attempted to Alter Game Option. Ignoring")
		#Change Difficulty
		elif (protocol == DIFFICULTY_EVENT_ID):
			pPlayer = gc.getPlayer(data1)
			pPlayer.setHandicap(data2)
		#Change Color
		elif (protocol == COLOR_EVENT_ID):
			pPlayer = gc.getPlayer(data1)
			pPlayer.setColor(data2)
			gc.getMap().updateMinimapColor()
		#disabled/enable worker actions
		elif (protocol == CANBUILD_EVENT_ID):
			if (data1 == 0):
				gc.getBuildInfo(data2).setDisabled(data3)
			else:
				BugUtil.debug("Non-Host Player Attempted to Alter Game Option. Ignoring")


##################################################		
# Module level functions defined in RoMSettings.xml	
##################################################	

def changedRoMReset (option, value):
	resetOptions()
	return True

########################################################################
def changedDefenderWithdraw(option, value):
	gc.getGame().setModderGameOption(ModderGameOptionTypes.MODDERGAMEOPTION_DEFENDER_WITHDRAW, value)
	CyMessageControl().sendModNetMessage(MODDERGAMEOPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderGameOptionTypes.MODDERGAMEOPTION_DEFENDER_WITHDRAW), int(value), 0)
def changedMaxUnitsPerTile(option, value):
	gc.getGame().setModderGameOption(ModderGameOptionTypes.MODDERGAMEOPTION_MAX_UNITS_PER_TILES, value)
	CyMessageControl().sendModNetMessage(MODDERGAMEOPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderGameOptionTypes.MODDERGAMEOPTION_MAX_UNITS_PER_TILES), int(value), 0)
def changedFlexibleDifficultyTurnIncrements(option, value):
	gc.getActivePlayer().setModderOption(ModderOptionTypes.MODDEROPTION_FLEXIBLE_DIFFICULTY_TURN_INCREMENTS, value)
	CyMessageControl().sendModNetMessage(MODDEROPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderOptionTypes.MODDEROPTION_FLEXIBLE_DIFFICULTY_TURN_INCREMENTS), int(value), 0)
def changedMaxBombardDefense(option, value):
	gc.getGame().setModderGameOption(ModderGameOptionTypes.MODDERGAMEOPTION_MAX_BOMBARD_DEFENSE, value)
	CyMessageControl().sendModNetMessage(MODDERGAMEOPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderGameOptionTypes.MODDERGAMEOPTION_MAX_BOMBARD_DEFENSE), int(value), 0)
def changedCanNotClaimOcean(option, value):
	gc.getGame().setModderGameOption(ModderGameOptionTypes.MODDERGAMEOPTION_CANNOT_CLAIM_OCEAN, value)
	CyMessageControl().sendModNetMessage(MODDERGAMEOPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderGameOptionTypes.MODDERGAMEOPTION_CANNOT_CLAIM_OCEAN), int(value), 0)
def changedShowCivTraits(option, value):
	gc.getActivePlayer().setModderOption(ModderOptionTypes.MODDEROPTION_SHOW_TRAITS_FLAG, value)
	CyMessageControl().sendModNetMessage(MODDEROPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderOptionTypes.MODDEROPTION_SHOW_TRAITS_FLAG), int(value), 0)
def changedNoFriendlyPillaging(option, value):
	gc.getActivePlayer().setModderOption(ModderOptionTypes.MODDEROPTION_NO_FRIENDLY_PILLAGING, value)
	CyMessageControl().sendModNetMessage(MODDEROPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderOptionTypes.MODDEROPTION_NO_FRIENDLY_PILLAGING), int(value), 0)
def changedEnableFlexibleDifficulty(option, value):
	gc.getActivePlayer().setModderOption(ModderOptionTypes.MODDEROPTION_FLEXIBLE_DIFFICULTY, value)
	CyMessageControl().sendModNetMessage(MODDEROPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderOptionTypes.MODDEROPTION_FLEXIBLE_DIFFICULTY), int(value), 0)
def changedFlexibleDifficultyMinimumDiff(option, value):
	if (value > 0):
		gc.getActivePlayer().setModderOption(ModderOptionTypes.MODDEROPTION_FLEXIBLE_DIFFICULTY_MIN_DIFFICULTY, value - 1)
		CyMessageControl().sendModNetMessage(MODDEROPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderOptionTypes.MODDEROPTION_FLEXIBLE_DIFFICULTY_MIN_DIFFICULTY), int(value - 1), 0)
	else:
		gc.getActivePlayer().setModderOption(ModderOptionTypes.MODDEROPTION_FLEXIBLE_DIFFICULTY_MIN_DIFFICULTY, -1)
		CyMessageControl().sendModNetMessage(MODDEROPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderOptionTypes.MODDEROPTION_FLEXIBLE_DIFFICULTY_MIN_DIFFICULTY), int(-1), 0)
def changedFlexibleDifficultyMaximumDiff(option, value):
	if (value > 0):
		gc.getActivePlayer().setModderOption(ModderOptionTypes.MODDEROPTION_FLEXIBLE_DIFFICULTY_MAX_DIFFICULTY, value - 1)
		CyMessageControl().sendModNetMessage(MODDEROPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderOptionTypes.MODDEROPTION_FLEXIBLE_DIFFICULTY_MAX_DIFFICULTY), int(value - 1), 0)
	else:
		gc.getActivePlayer().setModderOption(ModderOptionTypes.MODDEROPTION_FLEXIBLE_DIFFICULTY_MAX_DIFFICULTY, -1)
		CyMessageControl().sendModNetMessage(MODDEROPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderOptionTypes.MODDEROPTION_FLEXIBLE_DIFFICULTY_MAX_DIFFICULTY), int(-1), 0)
def changedFlexibleDifficultyMinRank(option, value):
	gc.getActivePlayer().setModderOption(ModderOptionTypes.MODDEROPTION_FLEXIBLE_DIFFICULTY_MIN_RANK, value)
	CyMessageControl().sendModNetMessage(MODDEROPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderOptionTypes.MODDEROPTION_FLEXIBLE_DIFFICULTY_MIN_RANK), int(value), 0)
def changedFlexibleDifficultyMaxRank(option, value):
	gc.getActivePlayer().setModderOption(ModderOptionTypes.MODDEROPTION_FLEXIBLE_DIFFICULTY_MAX_RANK, value)
	CyMessageControl().sendModNetMessage(MODDEROPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderOptionTypes.MODDEROPTION_FLEXIBLE_DIFFICULTY_MAX_RANK), int(value), 0)
def changedShowCoastalBuildings(option, value):
	gc.getActivePlayer().setModderOption(ModderOptionTypes.MODDEROPTION_SHOW_COASTAL_BUILDINGS, value)
	CyMessageControl().sendModNetMessage(MODDEROPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderOptionTypes.MODDEROPTION_SHOW_COASTAL_BUILDINGS), int(value), 0)
def changedHideObsoleteBuildings(option, value):
	gc.getActivePlayer().setModderOption(ModderOptionTypes.MODDEROPTION_HIDE_OBSOLETE_BUILDINGS, value)
	CyMessageControl().sendModNetMessage(MODDEROPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderOptionTypes.MODDEROPTION_HIDE_OBSOLETE_BUILDINGS), int(value), 0)
def changedDepletionMod(option, value):
	gc.getGame().setModderGameOption(ModderGameOptionTypes.MODDERGAMEOPTION_RESOURCE_DEPLETION, value)
	CyMessageControl().sendModNetMessage(MODDERGAMEOPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderGameOptionTypes.MODDERGAMEOPTION_RESOURCE_DEPLETION), int(value), 0)
def changedBetterAirInterception(option, value):
	gc.getGame().setModderGameOption(ModderGameOptionTypes.MODDERGAMEOPTION_BETTER_INTERCETION, value)
	CyMessageControl().sendModNetMessage(MODDERGAMEOPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderGameOptionTypes.MODDERGAMEOPTION_BETTER_INTERCETION), int(value), 0)
def changedMaxRebaseRange(option, value):
	gc.getGame().setModderGameOption(ModderGameOptionTypes.MODDERGAMEOPTION_AIRLIFT_RANGE, value)
	CyMessageControl().sendModNetMessage(MODDERGAMEOPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderGameOptionTypes.MODDERGAMEOPTION_AIRLIFT_RANGE), int(value), 0)
def changedMercyRule(option, value):
	gc.getGame().setModderGameOption(ModderGameOptionTypes.MODDERGAMEOPTION_MERCY_RULE, value)
	CyMessageControl().sendModNetMessage(MODDERGAMEOPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderGameOptionTypes.MODDERGAMEOPTION_MERCY_RULE), int(value), 0)
def changedRealisiticDiplomacy(option, value):
	gc.getGame().setModderGameOption(ModderGameOptionTypes.MODDERGAMEOPTION_REALISTIC_DIPLOMACY, value)
	CyMessageControl().sendModNetMessage(MODDERGAMEOPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderGameOptionTypes.MODDERGAMEOPTION_REALISTIC_DIPLOMACY), int(value), 0)
def changedShowRevCivics(option, value):
	gc.getActivePlayer().setModderOption(ModderOptionTypes.MODDEROPTION_SHOW_REV_EFFECTS, value)
	CyMessageControl().sendModNetMessage(MODDEROPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderOptionTypes.MODDEROPTION_SHOW_REV_EFFECTS), int(value), 0)
def changedBattlefieldPromotions(option, value):
	gc.getGame().setModderGameOption(ModderGameOptionTypes.MODDERGAMEOPTION_BATTLEFIELD_PROMOTIONS, value)
	CyMessageControl().sendModNetMessage(MODDERGAMEOPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderGameOptionTypes.MODDERGAMEOPTION_BATTLEFIELD_PROMOTIONS), int(value), 0)
def changedHideReplacedBuildings(option, value):
	gc.getActivePlayer().setModderOption(ModderOptionTypes.MODDEROPTION_HIDE_REPLACED_BUILDINGS, value)
	CyMessageControl().sendModNetMessage(MODDEROPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderOptionTypes.MODDEROPTION_HIDE_REPLACED_BUILDINGS), int(value), 0)
def changedNoStorms(option, value):
	gc.getGame().setModderGameOption(ModderGameOptionTypes.MODDERGAMEOPTION_NO_STORMS, value)
	CyMessageControl().sendModNetMessage(MODDERGAMEOPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderGameOptionTypes.MODDERGAMEOPTION_NO_STORMS), int(value), 0)
def changedImprovedXP(option, value):
	gc.getGame().setModderGameOption(ModderGameOptionTypes.MODDERGAMEOPTION_IMPROVED_XP, value)
	CyMessageControl().sendModNetMessage(MODDERGAMEOPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderGameOptionTypes.MODDERGAMEOPTION_IMPROVED_XP), int(value), 0)
def changedPlayerColor(option, value):
	iColor = value - 1
	if (iColor >= 0):
		#gc.getActivePlayer().setColor(iColor)
		CyMessageControl().sendModNetMessage(COLOR_EVENT_ID, gc.getGame().getActivePlayer(), iColor, 0, 0)
def updateAliveCivsOption():
	import BugGame
	aliveCivsOption = BugGame.ANewDawnSettings.AliveCivilization
	descs = []
	for player in PlayerUtil.players(True, True, False, False, False):
		descs.append(gc.getCivilizationInfo(player.getCivilizationType()).getShortDescription())
	sort(descs)
	aliveCivsOption.setValues(descs)
def changedCurrentDifficulty(option, value):
	iDifficulty = value - 1
	if (iDifficulty >= 0):
		CyMessageControl().sendModNetMessage(DIFFICULTY_EVENT_ID, gc.getGame().getActivePlayer(), iDifficulty, 0, 0)
		#gc.getActivePlayer().setHandicap(iDifficulty)
def changedUseLandmarkNames(option, value):
	gc.getActivePlayer().setModderOption(ModderOptionTypes.MODDEROPTION_USE_LANDMARK_NAMES, value)
	CyMessageControl().sendModNetMessage(MODDEROPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderOptionTypes.MODDEROPTION_USE_LANDMARK_NAMES), int(value), 0)
def changedHideUnavailableBuilds(option, value):
	gc.getActivePlayer().setModderOption(ModderOptionTypes.MODDEROPTION_HIDE_UNAVAILBLE_BUILDS, value)
	CyMessageControl().sendModNetMessage(MODDEROPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderOptionTypes.MODDEROPTION_HIDE_UNAVAILBLE_BUILDS), int(value), 0)
def changedHideObsoleteBuilds(option, value):
	gc.getActivePlayer().setModderOption(ModderOptionTypes.MODDEROPTION_HIDE_OBSOLETE_BUILDS, value)
	CyMessageControl().sendModNetMessage(MODDEROPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderOptionTypes.MODDEROPTION_HIDE_OBSOLETE_BUILDS), int(value), 0)
def changedEventImages(option, value):
	gc.getActivePlayer().setModderOption(ModderOptionTypes.MODDEROPTION_EVENT_IMAGES, value)
	CyMessageControl().sendModNetMessage(MODDEROPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderOptionTypes.MODDEROPTION_EVENT_IMAGES), int(value), 0)
def changedIgnoreDisabledBuildingAlerts(option, value):
	gc.getActivePlayer().setModderOption(ModderOptionTypes.MODDEROPTION_IGNORE_DISABLED_ALERTS, value)
	CyMessageControl().sendModNetMessage(MODDEROPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderOptionTypes.MODDEROPTION_IGNORE_DISABLED_ALERTS), int(value), 0)
def changedMultipleReligionSpread(option, value):
	gc.getGame().setModderGameOption(ModderGameOptionTypes.MODDERGAMEOPTION_MULTIPLE_RELIGION_SPREAD, value)
	CyMessageControl().sendModNetMessage(MODDERGAMEOPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderGameOptionTypes.MODDERGAMEOPTION_MULTIPLE_RELIGION_SPREAD), int(value), 0)
def changedTerrainDamage(option, value):
	gc.getGame().setModderGameOption(ModderGameOptionTypes.MODDERGAMEOPTION_TERRAIN_DAMAGE, value)
	CyMessageControl().sendModNetMessage(MODDERGAMEOPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderGameOptionTypes.MODDERGAMEOPTION_TERRAIN_DAMAGE), int(value), 0)
def changedStrategicEvents(option, value):
	gc.getGame().setModderGameOption(ModderGameOptionTypes.MODDERGAMEOPTION_STRATEGIC_EVENTS, value)
	CyMessageControl().sendModNetMessage(MODDERGAMEOPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderGameOptionTypes.MODDERGAMEOPTION_STRATEGIC_EVENTS), int(value), 0)
	
def changedAllowTerraforming(option, value):
	CyMessageControl().sendModNetMessage(CANBUILD_EVENT_ID, gc.getGame().getActivePlayer(), gc.getInfoTypeForString("BUILD_TERRAFORM_GRASS"), not value, 0)
	CyMessageControl().sendModNetMessage(CANBUILD_EVENT_ID, gc.getGame().getActivePlayer(), gc.getInfoTypeForString("BUILD_TERRAFORM_PLAINS"), not value, 0)
	CyMessageControl().sendModNetMessage(CANBUILD_EVENT_ID, gc.getGame().getActivePlayer(), gc.getInfoTypeForString("BUILD_TERRAFORM_TUNDRA"), not value, 0)
	
def changedReforestation(option, value):
	CyMessageControl().sendModNetMessage(CANBUILD_EVENT_ID, gc.getGame().getActivePlayer(), gc.getInfoTypeForString("BUILD_PLANT_FOREST"), not value, 0)
	CyMessageControl().sendModNetMessage(CANBUILD_EVENT_ID, gc.getGame().getActivePlayer(), gc.getInfoTypeForString("BUILD_MAKE_JUNGLE"), not value, 0)
	
def changedSeaTunnels(option, value):
	CyMessageControl().sendModNetMessage(CANBUILD_EVENT_ID, gc.getGame().getActivePlayer(), gc.getInfoTypeForString("BUILD_TUNNEL"), not value, 0)

def changedNoAutoCorporationFounding(option, value):
	gc.getGame().setModderGameOption(ModderGameOptionTypes.MODDERGAMEOPTION_NO_AUTO_CORPORATION_FOUNDING, value)
	CyMessageControl().sendModNetMessage(MODDERGAMEOPTION_EVENT_ID, gc.getGame().getActivePlayer(), int(ModderGameOptionTypes.MODDERGAMEOPTION_NO_AUTO_CORPORATION_FOUNDING), int(value), 0)
	
def changedWarPrizes(option, value):
	gc.setDefineINT("WAR_PRIZES", int(value))


def setXMLOptionsfromIniFile():
	BugUtil.debug("Initializing A New Dawn Settings")

	changedDefenderWithdraw(ANewDawnOpt, ANewDawnOpt.isDefenderWithdraw())
	changedMaxUnitsPerTile(ANewDawnOpt, ANewDawnOpt.getMaxUnitsPerTile())
	changedFlexibleDifficultyTurnIncrements(ANewDawnOpt, ANewDawnOpt.getFlexibleDifficultyTurnIncrements())
	changedMaxBombardDefense(ANewDawnOpt, ANewDawnOpt.getMaxBombardDefense())
	changedCanNotClaimOcean(ANewDawnOpt, ANewDawnOpt.isCanNotClaimOcean())
	changedShowCivTraits(ANewDawnOpt, ANewDawnOpt.isShowCivTraits())
	changedNoFriendlyPillaging(ANewDawnOpt, ANewDawnOpt.isNoFriendlyPillaging())
	changedEnableFlexibleDifficulty(ANewDawnOpt, ANewDawnOpt.isEnableFlexibleDifficulty())
	changedFlexibleDifficultyMinimumDiff(ANewDawnOpt, ANewDawnOpt.getFlexibleDifficultyMinimumDiff() - 1)
	changedFlexibleDifficultyMaximumDiff(ANewDawnOpt, ANewDawnOpt.getFlexibleDifficultyMaximumDiff() - 1)
	changedFlexibleDifficultyMinRank(ANewDawnOpt, ANewDawnOpt.getFlexibleDifficultyMinRank())
	changedFlexibleDifficultyMaxRank(ANewDawnOpt, ANewDawnOpt.getFlexibleDifficultyMaxRank())
	changedShowCoastalBuildings(ANewDawnOpt, ANewDawnOpt.isShowCoastalBuildings())
	changedHideObsoleteBuildings(ANewDawnOpt, ANewDawnOpt.isHideObsoleteBuildings())
	changedDepletionMod(ANewDawnOpt, ANewDawnOpt.isDepletionMod())
	changedBetterAirInterception(ANewDawnOpt, ANewDawnOpt.isBetterAirInterception())
	changedMaxRebaseRange(ANewDawnOpt, ANewDawnOpt.getMaxRebaseRange())
	changedMercyRule(ANewDawnOpt, ANewDawnOpt.isMercyRule())
	changedRealisiticDiplomacy(ANewDawnOpt, ANewDawnOpt.isRealisiticDiplomacy())
	changedShowRevCivics(ANewDawnOpt, ANewDawnOpt.isShowRevCivics())
	changedBattlefieldPromotions(ANewDawnOpt, ANewDawnOpt.isBattlefieldPromotions())
	changedHideReplacedBuildings(ANewDawnOpt, ANewDawnOpt.isHideReplacedBuildings())
	changedNoStorms(ANewDawnOpt, ANewDawnOpt.isNoStorms())
	changedImprovedXP(ANewDawnOpt, ANewDawnOpt.isImprovedXP())
	changedUseLandmarkNames(ANewDawnOpt, ANewDawnOpt.isUseLandmarkNames())
	changedHideUnavailableBuilds(ANewDawnOpt, ANewDawnOpt.isHideUnavailableBuilds())
	changedHideObsoleteBuilds(ANewDawnOpt, ANewDawnOpt.isHideObsoleteBuilds())
	changedEventImages(ANewDawnOpt, ANewDawnOpt.isEventImages())
	changedIgnoreDisabledBuildingAlerts(ANewDawnOpt, ANewDawnOpt.isIgnoreDisabledBuildingAlerts())
	changedMultipleReligionSpread(ANewDawnOpt, ANewDawnOpt.isMultipleReligionSpread())
	changedTerrainDamage(ANewDawnOpt, ANewDawnOpt.isTerrainDamage())
	changedStrategicEvents(ANewDawnOpt, ANewDawnOpt.isStrategicEvents())
	changedAllowTerraforming(ANewDawnOpt, ANewDawnOpt.isAllowTerraforming())
	changedReforestation(ANewDawnOpt, ANewDawnOpt.isReforestation())
	changedSeaTunnels(ANewDawnOpt, ANewDawnOpt.isSeaTunnels())
	changedWarPrizes(ANewDawnOpt, ANewDawnOpt.isWarPrizes())
	
	ANewDawnOpt.setPlayerColor(0)
	ANewDawnOpt.setCurrentDifficulty(0)
	
def resetOptions():
	ANewDawnoptions = BugOptions.getOptions("RoMSettings").options
	for i in range(len(ANewDawnoptions)):
		ANewDawnoptions[i].resetValue()
	setXMLOptionsfromIniFile()
	ANewDawnOpt.setRoMReset(False)